#ifndef __DNS2_H__
#define __DNS2_H__

#include <stdio.h>
#include <stdlib.h>

#include <time.h>

#include "rr.h"

enum {
	DNS_RESOLVER = 0,
	DNS_UPDATER
};

enum {
	DNS_STATE_FREE = 0,
	DNS_STATE_UDP,
	DNS_STATE_TCP_CONNECT,
	DNS_STATE_TCP
};

enum {
	DNS_COMMAND_ERROR_UNSIGNED = -2,
	DNS_COMMAND_ERROR = -1,
	DNS_COMMAND_POLL = 0,
	DNS_COMMAND_SUCCESS = 1
};

#define DNS_POLL_WANT_NONE 0x00
#define DNS_POLL_WANT_READ 0x01
#define DNS_POLL_WANT_WRITE 0x02

typedef struct _DNS2 DNS2;

DNS2 *dns2_resolver_new (int recursion);
void dns2_resolver_set_question (DNS2 *resolver, const char *name, int type, int klass);

DNS2 *dns2_updater_new (const char *zone);
void dns2_updater_add (DNS2 *updater, DNSRR *rr);

void dns2_add_nserver (DNS2 *obj, const char *ip);
void dns2_free (DNS2 *obj);

void dns2_sign_tsig (DNS2 *obj, const char *keyname, const char *signature, const char *algorithm);

int dns2_sync (DNS2 *obj);
int dns2_async (DNS2 *obj);
int dns2_async_continue (DNS2 *obj, int happen);

int dns2_check_timeout (DNS2 *obj);
int dns2_get_socket (DNS2 *obj);
int dns2_get_socket_poll_flags (DNS2 *obj);
void dns2_set_use_tcp (DNS2 *obj);

DNSRR **dns2_get_answers_from_response (DNS2 *obj);
DNSRR **dns2_get_authority_from_response (DNS2 *obj);
DNSRR **dns2_get_additional_from_response (DNS2 *obj);

int dns2_get_rcode (DNS2 *obj);

#endif /* __DNS2_H__ */

